@@ -2,7 +2,7 @@
 #this repository contains the full copyright notices and license terms.
 from decimal import Decimal
 
-from trytond.model import fields
+from trytond.model import Workflow, ModelView, fields
 from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
 
@@ -22,6 +22,7 @@ class Move:
         pool = Pool()
         Uom = pool.get('product.uom')
         AccountMoveLine = pool.get('account.move.line')
+        Currency = pool.get('currency.currency')
         assert type_.startswith('in_') or type_.startswith('out_'), \
             'wrong type'
 
@@ -30,7 +31,9 @@ class Move:
         if ((type_.endswith('supplier')
                     or type_ == 'in_production')
                 and self.product.cost_price_method != 'fixed'):
-            unit_price = self.unit_price
+            with Transaction().set_context(date=self.effective_date):
+                unit_price = Currency.compute(self.currency, self.unit_price,
+                    self.company.currency, round=False)
         else:
             unit_price = Uom.compute_price(self.product.default_uom,
                 self.cost_price, self.uom)
@@ -168,15 +171,9 @@ class Move:
         return super(Move, cls).copy(moves, default=default)
 
     @classmethod
-    def create(cls, vals):
-        move = super(Move, cls).create(vals)
-        if vals.get('state') == 'done':
+    @ModelView.button
+    @Workflow.transition('done')
+    def do(cls, moves):
+        super(Move, cls).do(moves)
+        for move in moves:
             move._create_account_stock_move()
-        return move
-
-    @classmethod
-    def write(cls, moves, vals):
-        super(Move, cls).write(moves, vals)
-        if vals.get('state') == 'done':
-            for move in moves:
-                move._create_account_stock_move()
